@@ -2,11 +2,12 @@ from __future__ import annotations
 
 import ctypes
 import math
+from typing import Sequence
 
 import jax
 import jax.numpy as jnp
 import requests
-from jax import tree_flatten
+from jax.tree_util import tree_flatten
 
 from .tree_util import (
     _reduce_count_and_size,
@@ -37,7 +38,7 @@ def _format_count(node_count, newline=False):
     return f"{int(node_count.real):,}{mark}({int(node_count.imag):,})"
 
 
-def _node_format(node):
+def _format_node(node):
     """format shape and dtype of jnp.array"""
 
     if isinstance(node, (jnp.ndarray, jax.ShapeDtypeStruct)):
@@ -45,6 +46,7 @@ def _node_format(node):
             ("int", "i"),
             ("float", "f"),
             ("complex", "c"),
+            (",)", ")"),
             ("(", "["),
             (")", "]"),
             (" ", ""),
@@ -52,7 +54,6 @@ def _node_format(node):
 
         formatted_string = f"{node.dtype}{jnp.shape(node)!r}"
 
-        # trunk-ignore
         for lhs, rhs in replace_tuple:
             formatted_string = formatted_string.replace(lhs, rhs)
         return formatted_string
@@ -95,21 +96,21 @@ def _hstack(boxes):
     return fmt
 
 
-def _vbox(*text):
-    """
-    === Explanation
-        create vertically stacked text boxes
+def _vbox(*text: tuple[str, ...]) -> str:
+    """Create vertically stacked text boxes
 
-    === Examples
+    Returns:
+        str: stacked boxes string
 
-        >> _vbox("a","b")
+    Examples:
+    >>> _vbox("a","b")
         ┌───┐
         │a  │
         ├───┤
         │b  │
         └───┘
 
-        >> _vbox("a","","a")
+    >>> _vbox("a","","a")
         ┌───┐
         │a  │
         ├───┤
@@ -147,12 +148,16 @@ def _vbox(*text):
     return formatted
 
 
-def _resolve_line(cols):
-    """
-    === Explanation
-        combine columns of single line by merging their borders
+def _resolve_line(cols: Sequence[str, ...]) -> str:
+    """combine columns of single line by merging their borders
 
-    === Examples
+    Args:
+        cols (Sequence[str,...]): Sequence of single line column string
+
+    Returns:
+        str: resolved column string
+
+    Example:
         >>> _resolve_line(['ab','b│','│c'])
         'abb│c'
 
@@ -261,7 +266,7 @@ def _layer_box(name, indim=None, outdim=None):
 
 def _summary_line(leaf):
 
-    dynamic, static = leaf.tree_fields
+    dynamic, static = leaf.__tree_fields__
     is_dynamic = not leaf.frozen
     class_name = leaf.__class__.__name__
 
@@ -284,6 +289,8 @@ def _summary_str(model, array=None, render: str = "string") -> str:
 
     ROW = [["Type ", "Param #", "Size ", "Config", "Output"]]
 
+    excluded_kw = ["__frozen_treeclass__", "__frozen_tree_fields__"]
+
     dynamic_count, static_count = 0, 0
     dynamic_size, static_size = 0, 0
 
@@ -291,22 +298,25 @@ def _summary_str(model, array=None, render: str = "string") -> str:
         params_shape = sequential_model_shape_eval(model, array)[1:]
 
     # all dynamic/static leaves
-    all_leaves = (*model.tree_fields[0].values(), *model.tree_fields[1].values())
+    all_leaves = (
+        *model.__tree_fields__[0].values(),
+        *model.__tree_fields__[1].values(),
+    )
     treeclass_leaves = [leaf for leaf in all_leaves if is_treeclass(leaf)]
 
     for index, leaf in enumerate(treeclass_leaves):
         name, count, size = _summary_line(leaf)
 
-        shape = _node_format(params_shape[index]) if array is not None else ""
+        shape = _format_node(params_shape[index]) if array is not None else ""
 
         if leaf.frozen:
             static_count += count
             static_size += size
             fmt = "\n".join(
                 [
-                    f"{k}={_node_format(v)}"
-                    for k, v in leaf.tree_fields[1].items()
-                    if k != "__frozen_treeclass__"
+                    f"{k}={_format_node(v)}"
+                    for k, v in leaf.__tree_fields__[1].items()
+                    if k not in excluded_kw
                 ]
             )
 
@@ -314,7 +324,7 @@ def _summary_str(model, array=None, render: str = "string") -> str:
             dynamic_count += count
             dynamic_size += size
             fmt = "\n".join(
-                [f"{k}={_node_format(v)}" for k, v in leaf.tree_fields[0].items()]
+                [f"{k}={_format_node(v)}" for k, v in leaf.__tree_fields__[0].items()]
             )
 
         ROW += [
@@ -359,6 +369,8 @@ def _summary_md(model, array=None) -> str:
         "</tr>\n"
     )
 
+    excluded_kw = ["__frozen_treeclass__", "__frozen_tree_fields__"]
+
     dynamic_count, static_count = 0, 0
     dynamic_size, static_size = 0, 0
 
@@ -366,22 +378,25 @@ def _summary_md(model, array=None) -> str:
         params_shape = sequential_model_shape_eval(model, array)[1:]
 
     # all dynamic/static leaves
-    all_leaves = (*model.tree_fields[0].values(), *model.tree_fields[1].values())
+    all_leaves = (
+        *model.__tree_fields__[0].values(),
+        *model.__tree_fields__[1].values(),
+    )
     treeclass_leaves = [leaf for leaf in all_leaves if is_treeclass(leaf)]
 
     for index, leaf in enumerate(treeclass_leaves):
         name, count, size = _summary_line(leaf)
 
-        shape = _node_format(params_shape[index]) if array is not None else ""
+        shape = _format_node(params_shape[index]) if array is not None else ""
 
         if leaf.frozen:
             static_count += count
             static_size += size
             config = "<br>".join(
                 [
-                    f"{k}={_node_format(v)}"
-                    for k, v in leaf.tree_fields[1].items()
-                    if k != "__frozen_treeclass__"
+                    f"{k}={_format_node(v)}"
+                    for k, v in leaf.__tree_fields__[1].items()
+                    if k not in excluded_kw
                 ]
             )
 
@@ -389,7 +404,7 @@ def _summary_md(model, array=None) -> str:
             dynamic_count += count
             dynamic_size += size
             config = "<br>".join(
-                [f"{k}={_node_format(v)}" for k, v in leaf.tree_fields[0].items()]
+                [f"{k}={_format_node(v)}" for k, v in leaf.__tree_fields__[0].items()]
             )
 
         fmt += (
@@ -450,8 +465,8 @@ def tree_box(model, array=None):
         if is_treeclass_leaf(model):
             box = _layer_box(
                 f"{model.__class__.__name__}({parent_name})",
-                _node_format(shapes[0]) if array is not None else None,
-                _node_format(shapes[1]) if array is not None else None,
+                _format_node(shapes[0]) if array is not None else None,
+                _format_node(shapes[1]) if array is not None else None,
             )
 
             if shapes is not None:
@@ -461,14 +476,14 @@ def tree_box(model, array=None):
         else:
             level_nodes = []
 
-            for field in model.__dataclass_fields__.values():
-                cur_node = model.__dict__[field.name]
+            for fi in model.__dataclass_fields__.values():
+                cur_node = model.__dict__[fi.name]
 
                 if is_treeclass(cur_node):
-                    level_nodes += [f"{recurse(cur_node,field.name)}"]
+                    level_nodes += [f"{recurse(cur_node,fi.name)}"]
 
                 else:
-                    level_nodes += [_vbox(f"{field.name}={_node_format(cur_node)}")]
+                    level_nodes += [_vbox(f"{fi.name}={_format_node(cur_node)}")]
 
             return _vbox(
                 f"{model.__class__.__name__}({parent_name})", "\n".join(level_nodes)
@@ -496,29 +511,41 @@ def tree_diagram(model):
             cur_children_count = len(model.__dataclass_fields__)
 
             for i, fi in enumerate(model.__dataclass_fields__.values()):
-                cur_node = model.__dict__[fi.name]
 
-                fmt += "\n" + "".join(
-                    [(("│" if lvl > 1 else "") + "\t") for lvl in parent_level_count]
-                )
+                if fi.repr:
+                    cur_node = model.__dict__[fi.name]
 
-                is_static = "static" in fi.metadata and fi.metadata["static"]
-                mark = "x" if is_static else ("#" if model.frozen else "─")
-
-                if is_treeclass(cur_node):
-
-                    layer_class_name = cur_node.__class__.__name__
-
-                    fmt += (
-                        f"├{mark}─ " if i < (cur_children_count - 1) else f"└{mark}─ "
-                    ) + f"{fi.name}={layer_class_name}"
-                    recurse(cur_node, parent_level_count + [cur_children_count - i])
-
-                else:
-                    fmt += (
-                        f"├{mark}─ " if i < (cur_children_count - 1) else f"└{mark}─ "
+                    fmt += "\n" + "".join(
+                        [
+                            (("│" if lvl > 1 else "") + "\t")
+                            for lvl in parent_level_count
+                        ]
                     )
-                    fmt += f"{fi.name}={_node_format(cur_node)}"
+
+                    is_static = "static" in fi.metadata and fi.metadata["static"]
+                    mark = "x" if is_static else ("#" if model.frozen else "─")
+
+                    if is_treeclass(cur_node):
+
+                        layer_class_name = cur_node.__class__.__name__
+
+                        fmt += (
+                            f"├{mark}─ "
+                            if i < (cur_children_count - 1)
+                            else f"└{mark}─ "
+                        ) + f"{fi.name}={layer_class_name}"
+                        recurse(cur_node, parent_level_count + [cur_children_count - i])
+
+                    else:
+                        fmt += (
+                            f"├{mark}─ "
+                            if i < (cur_children_count - 1)
+                            else f"└{mark}─ "
+                        )
+                        fmt += f"{fi.name}={_format_node(cur_node)}"
+                        recurse(cur_node, parent_level_count + [1])
+
+                elif not is_treeclass(model):
                     recurse(cur_node, parent_level_count + [1])
 
             fmt += "\t"
@@ -530,116 +557,143 @@ def tree_diagram(model):
     return fmt.expandtabs(4)
 
 
-def tree_indent(model):
-    """
-    === Explanation
-        pretty print treeclass model with indentation
+def tree_indent(model, width: int = 40) -> str:
+    """Prertty print `treeclass_leaves`
 
-    === Args
-        tree : boolean to create tree-structure
+    Returns:
+        str: indented model leaves.
     """
 
-    def recurse(model, parent_level_count):
+    def format_width(string, width=width):
+        """strip newline/tab characters if less than max width"""
+        stripped_string = string.replace("\n", "").replace("\t", "")
+        children_length = len(stripped_string)
+        return string if children_length > width else stripped_string
+
+    def recurse(model, depth):
 
         nonlocal fmt
 
         if is_treeclass(model):
             cur_children_count = len(model.__dataclass_fields__)
 
-            newline = cur_children_count > 1
+            for i, fi in enumerate(model.__dataclass_fields__.values()):
 
-            for i, field in enumerate(model.__dataclass_fields__.values()):
-                cur_node = model.__dict__[field.name]
-                fmt += "\n" + "\t" * len(parent_level_count) if newline else ""
+                if fi.repr:
+                    cur_node = model.__dict__[fi.name]
 
-                if is_treeclass(cur_node):
+                    # add newline by default
+                    fmt += ("\n" + "\t" * depth) if fi.repr else ""
 
-                    layer_class_name = f"{cur_node.__class__.__name__}"
-                    fmt += f"{field.name}={layer_class_name}" + "("
-                    recurse(cur_node, parent_level_count + [cur_children_count - i])
-                    fmt += ")"
+                    if is_treeclass(cur_node):
+                        layer_class_name = f"{cur_node.__class__.__name__}"
+                        fmt += f"{fi.name}={layer_class_name}" + "("
 
-                else:
-                    fmt += f"{field.name}={_node_format(cur_node)}" + (
-                        "," if i < (cur_children_count - 1) else ("")
-                    )
+                        # capture children repr
+                        start_cursor = len(fmt)
 
-                    recurse(cur_node, parent_level_count + [1])
+                        recurse(cur_node, depth + 1)
+
+                        # format children repr width
+                        fmt = fmt[:start_cursor] + format_width(fmt[start_cursor:])
+
+                        fmt += ")" + ("," if i < (cur_children_count - 1) else "")
+
+                    else:
+                        fmt += f"{fi.name}={_format_node(cur_node)}" + (
+                            "," if i < (cur_children_count - 1) else ("")
+                        )
+                        recurse(cur_node, depth)
+
+                elif not is_treeclass(model):
+                    recurse(cur_node, depth)
 
     fmt = ""
-    # fmt = f"{(model.__class__.__name__)}("
-    recurse(model, [1])
-
-    fmt = f"{(model.__class__.__name__)}({fmt})"
+    recurse(model, 1)
+    fmt = f"{(model.__class__.__name__)}({format_width(fmt,width)})"
 
     return fmt.expandtabs(2)
 
 
-def tree_str(model):
-    """
-    === Explanation
-        pretty print treeclass model with indentation
+def tree_str(model, width: int = 40) -> str:
+    """Prertty print `treeclass_leaves`
 
-    === Args
-        tree : boolean to create tree-structure
+    Returns:
+        str: indented model leaves.
     """
 
-    def recurse(model, parent_level_count):
+    def format_width(string, width=width):
+        """strip newline/tab characters if less than max width"""
+        stripped_string = string.replace("\n", "").replace("\t", "")
+        children_length = len(stripped_string)
+        return string if children_length > width else stripped_string
+
+    def recurse(model, depth):
 
         nonlocal fmt
 
         if is_treeclass(model):
             cur_children_count = len(model.__dataclass_fields__)
 
-            newline = cur_children_count > 1
+            for i, fi in enumerate(model.__dataclass_fields__.values()):
 
-            for i, field in enumerate(model.__dataclass_fields__.values()):
-                cur_node = model.__dict__[field.name]
-                fmt += "\n" + "\t" * len(parent_level_count) if newline else ""
+                if fi.repr:
+                    cur_node = model.__dict__[fi.name]
 
-                if is_treeclass(cur_node):
-                    layer_class_name = f"{cur_node.__class__.__name__}"
-                    fmt += f"{field.name}={layer_class_name}" + "("
-                    recurse(cur_node, parent_level_count + [cur_children_count - i])
-                    fmt += ")"
+                    # add newline by default
+                    fmt += ("\n" + "\t" * depth) if fi.repr else ""
 
-                else:
-                    formatted_str = (
-                        "\t" * (len(parent_level_count) + 1) + f"{cur_node!s}"
-                    )
-                    formatted_str = ("\n" + "\t" * (len(parent_level_count) + 1)).join(
-                        formatted_str.split("\n")
-                    )
-                    fmt += (
-                        f"{field.name}=\n"
-                        + formatted_str
-                        + ("," if i < (cur_children_count - 1) else "")
-                    )
+                    if is_treeclass(cur_node):
+                        layer_class_name = f"{cur_node.__class__.__name__}"
+                        fmt += f"{fi.name}={layer_class_name}" + "("
 
-                    recurse(cur_node, parent_level_count + [1])
+                        # capture children repr
+                        start_cursor = len(fmt)
+
+                        recurse(cur_node, depth + 1)
+
+                        # format children repr width
+                        fmt = fmt[:start_cursor] + format_width(fmt[start_cursor:])
+
+                        fmt += ")" + ("," if i < (cur_children_count - 1) else "")
+
+                    else:
+                        multiline = "\n" in f"{cur_node!s}"
+                        cur_node_fmt = ("\n" + "\t" * (depth + 1)) if multiline else ""
+                        cur_node_fmt += ("\n" + "\t" * (depth + 1)).join(
+                            f"{cur_node!s}".split("\n")
+                        )
+
+                        fmt += f"{fi.name}={cur_node_fmt}" + (
+                            "," if i < (cur_children_count - 1) else ("")
+                        )
+                        recurse(cur_node, depth)
+
+                elif not is_treeclass(model):
+                    recurse(cur_node, depth)
 
     fmt = ""
-    recurse(model, [1])
-    fmt = f"{(model.__class__.__name__)}({fmt})"
+    recurse(model, 1)
+    fmt = f"{(model.__class__.__name__)}({format_width(fmt,width)})"
 
     return fmt.expandtabs(2)
 
 
-def tree_mermaid(model, link=False):
-    def _tree_mermaid(model):
-        def node_id(input):
-            """hash a node by its location in a tree"""
-            return ctypes.c_size_t(hash(input)).value
+def _tree_mermaid(model):
+    def node_id(input):
+        """hash a node by its location in a tree"""
+        return ctypes.c_size_t(hash(input)).value
 
-        def recurse(model, cur_depth, prev_id):
+    def recurse(model, cur_depth, prev_id):
 
-            nonlocal fmt
+        nonlocal fmt
 
-            if is_treeclass(model):
-                is_frozen = model.frozen
+        if is_treeclass(model):
+            is_frozen = model.frozen
 
-                for i, field in enumerate(model.__dataclass_fields__.values()):
-                    cur_node = model.__dict__[field.name]
+            for i, fi in enumerate(model.__dataclass_fields__.values()):
+                if fi.repr:
+                    cur_node = model.__dict__[fi.name]
                     cur_order = i
                     fmt += "\n"
 
@@ -647,36 +701,40 @@ def tree_mermaid(model, link=False):
                         layer_class_name = cur_node.__class__.__name__
                         cur = (cur_depth, cur_order)
                         cur_id = node_id((*cur, prev_id))
-                        fmt += f"\tid{prev_id} --> id{cur_id}({field.name}\\n{layer_class_name})"
+                        fmt += f"\tid{prev_id} --> id{cur_id}({fi.name}\\n{layer_class_name})"
                         recurse(cur_node, cur_depth + 1, cur_id)
 
                     else:
                         cur = (cur_depth, cur_order)
                         cur_id = node_id((*cur, prev_id))
-                        is_static = (
-                            "static" in field.metadata and field.metadata["static"]
-                        )
+                        is_static = "static" in fi.metadata and fi.metadata["static"]
                         connector = (
                             "--x" if is_static else ("-.-" if is_frozen else "---")
                         )
-                        fmt += f'\tid{prev_id} {connector} id{cur_id}["{field.name}\\n{_node_format(cur_node)}"]'
+                        fmt += f'\tid{prev_id} {connector} id{cur_id}["{fi.name}\\n{_format_node(cur_node)}"]'
                         recurse(cur_node, cur_depth + 1, cur_id)
 
-        cur_id = node_id((0, 0, -1, 0))
-        fmt = f"flowchart TD\n\tid{cur_id}[{model.__class__.__name__}]"
-        recurse(model, 1, cur_id)
-        return fmt.expandtabs(4)
+                elif not is_treeclass(model):
+                    recurse(cur_node, cur_depth + 1, cur_id)
 
-    def generate_link(model) -> str:
-        """generate a one-time link mermaid diagram"""
-        url_val = "https://pytreeclass.herokuapp.com/generateTemp"
-        mermaid_string = _tree_mermaid(model)
-        request = requests.post(url_val, json={"description": mermaid_string})
-        generated_id = request.json()["id"]
-        generated_html = f"https://pytreeclass.herokuapp.com/temp/?id={generated_id}"
-        return f"Open URL in browser: {generated_html}"
+    cur_id = node_id((0, 0, -1, 0))
+    fmt = f"flowchart LR\n\tid{cur_id}[{model.__class__.__name__}]"
+    recurse(model, 1, cur_id)
+    return fmt.expandtabs(4)
 
-    return generate_link(model) if link else _tree_mermaid(model)
+
+def _generate_mermaid_link(mermaid_string: str) -> str:
+    """generate a one-time link mermaid diagram"""
+    url_val = "https://pytreeclass.herokuapp.com/generateTemp"
+    request = requests.post(url_val, json={"description": mermaid_string})
+    generated_id = request.json()["id"]
+    generated_html = f"https://pytreeclass.herokuapp.com/temp/?id={generated_id}"
+    return f"Open URL in browser: {generated_html}"
+
+
+def tree_mermaid(model, link=False):
+    mermaid_string = _tree_mermaid(model)
+    return _generate_mermaid_link(mermaid_string) if link else mermaid_string
 
 
 def save_viz(model, filename, method="tree_mermaid_md"):
