@@ -1,10 +1,11 @@
 import jax.numpy as jnp
+import pytest
 
 from pytreeclass import treeclass
 from pytreeclass.src.tree_util import (
+    _node_count_and_size,
     is_treeclass,
     is_treeclass_leaf,
-    _node_count_and_size,
 )
 
 
@@ -44,3 +45,26 @@ def test__node_count_and_size():
     t = Test()
     assert _node_count_and_size(t.b) == (complex(0, 1), complex(28, 0))
     assert _node_count_and_size(t.a) == (complex(3, 0), complex(12, 0))
+
+    assert _node_count_and_size(jnp.array([1, 2, 3, 4, 5])) == (
+        complex(0, 5),
+        complex(0, 20),
+    )
+    assert _node_count_and_size(3.0) == (complex(1), complex(24))
+
+    @treeclass
+    class x:
+        a: int = 1
+        b: int = 2
+
+    assert x().treeclass_leaves == [x()]
+
+    assert hash(x())
+
+    xx = x()
+    xx.cc = 1
+    assert xx.cc == 1
+    xx = xx.freeze()
+
+    with pytest.raises(ValueError):
+        xx.test = 1
